;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Picarus - Copyright 2005-2007 by Michael Kohn
;
; email: mike@mikekohn.net
; homepage: http://www.mikekohn.net/
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  include "p16f873.inc"

  PAGE
  __CONFIG _CP_OFF & _WDT_OFF & _XT_OSC & _PWRTE_ON 

  RADIX dec

; Set up some variables for storage

_status equ 32; 0x20
_w equ 33; 0x21
_pclath equ 34; 0x22

Temp equ 35; 0x23     ;
Servo equ 36; 0x24    ; 4.096 ms clocks
Servo_1 equ 37; 0x25
Servo_2 equ 38; 0x26
Servo_3 equ 39; 0x27
Servo_4 equ 40; 0x28
Servo_5 equ 41; 0x29

TICKS_BREAK equ 42; 0x2a
next_servo equ 43; 0x29

HexTemp equ 44; 0x2a
HexTemp2 equ 45; 0x2b

  org 0
  goto main

; Here's the interrupt routine for sending 1ms to 2ms pulses to
; the servos

  org 4

; store the current w and status flag so we can play with these
; registers

  movwf _w
  movf STATUS, w
  bcf STATUS, RP0
  bcf STATUS, RP1
  movwf _status 
  ; movf PCLATH, w
  ; movwf _pclath
  ; clrf PCLATH

  ; the Servo variable counts so that every 5th time this interrupt
  ; is called, the servo motors are serviced.  5 * 4.096ms = 20.48ms

  decfsz Servo, f 
  goto exitint

  ; Turn on all servos

  movlw 128 | 64 | 32 | 16 | 8
  iorwf PORTB, f

;  63 timer ticks = left     (0x3f)
;  94 timer ticks = center   (0x5e)
; 125 timer ticks = right    (0x7d)

intloop:
  ; Note:  TMR0 counts UPWARD

  ; movf TMR0, w
  ; subwf TICKS_BREAK, w
  ; btfss STATUS, C
  btfsc TMR0, 7
  goto exitintloop

  movf TMR0, w
  subwf Servo_1, w
  btfsc STATUS, C
  goto skip_servo1
  bcf PORTB, 7

skip_servo1:

  movf TMR0, w
  subwf Servo_2, w
  btfsc STATUS, C
  goto skip_servo2
  bcf PORTB, 6

skip_servo2:

  movf TMR0, w
  subwf Servo_3, w
  btfsc STATUS, C
  goto skip_servo3
  bcf PORTB, 5

skip_servo3:

  movf TMR0, w
  subwf Servo_4, w
  btfsc STATUS, C
  goto skip_servo4
  bcf PORTB, 4

skip_servo4:

  ;movf TMR0, w
  ;subwf Servo_5, w
  ;btfsc STATUS, C
  ;goto skip_servo5
  ;bcf PORTB, 3

;skip_servo5:

  goto intloop

exitintloop:

  movlw 5
  movwf Servo

  movlw 6
  andwf PORTB, f 

exitint:

; restore w and status to it's previous state

  ; movf _pclath, w
  ; movwf PCLATH
  movf _status, w
  movwf STATUS
  swapf _w, f
  swapf _w, w

  bcf INTCON, T0IF
  retfie

; function to write a single char out to the serial port

writechar:
  bsf STATUS, RP0
waitchar:
  btfss TXSTA ^ 0x080, TRMT
  goto waitchar
  bcf STATUS, RP0
  movwf TXREG
  return

output_hex:
  movwf HexTemp
  movwf HexTemp2
  rrf HexTemp2, f 
  rrf HexTemp2, f
  rrf HexTemp2, f
  rrf HexTemp2, w
  andlw 15
  movwf HexTemp2

  sublw 9
  btfss STATUS, C
  goto more_than_9
  movf HexTemp2, w
  addlw '0'
  call writechar 
  goto HEX_LSB
more_than_9: 
  movf HexTemp2, w
  addlw 'A'-10
  call writechar 

HEX_LSB:
  movf HexTemp, w
  andlw 15
  movwf HexTemp2

  sublw 9
  btfss STATUS, C
  goto more_than_9_lsb
  movf HexTemp2, w
  addlw '0'
  call writechar 
  return 
more_than_9_lsb:
  movf HexTemp2, w
  addlw 'A'-10
  call writechar 
  return

; main()

main:
  bsf STATUS, RP0

  movlw 131   ; controls the timing ( 1:16 = 0.016ms tick, 4.096ms int)
  movwf OPTION_REG ^ 0x80

  movlw 0
  movwf (TRISB ^ 0x080)    ; set the entire PORTB as outputs

  movlw (1 << TXEN)
  movwf (TXSTA ^ 0x80)
  movlw 0x33
  movwf SPBRG ^ 0x80
  bcf STATUS, RP0

  movlw (1 << SPEN) | (1 << CREN)
  movwf RCSTA

  bcf PORTB, 1
  bsf PORTB, 2

  movlw 5        ; 4.096ms * 5 = ~20ms
  movwf Servo
  movlw 94       ; set all servos to center position
  movwf Servo_1
  movwf Servo_2
  movwf Servo_3
  movwf Servo_4
  movwf Servo_5
  movlw 130         ; break after 130 timer ticks
  movwf TICKS_BREAK

  movlw 255         ; when a command comnes in, set this to server number
  movwf next_servo

  movlw 'P'         ; just output something over rs232 so we know it's alive
  call writechar
  movlw 'i'
  call writechar
  movlw 'c'
  call writechar
  movlw 'a'
  call writechar
  movlw 'r'
  call writechar
  movlw 'u'
  call writechar
  movlw 's'
  call writechar
  movlw '\r'
  call writechar
  movlw '\n'
  call writechar

  ; turn interrupts on
  clrf TMR0
  movlw (1<<GIE) | (1<<T0IE)
  movwf INTCON

MainLoop:
  btfss PIR1, RCIF
  goto MainLoop

  ; movlw 4
  movlw 2
  xorwf PORTB, f

  movf RCREG, w
  bcf PIR1, RCIF

; debug code
;  movwf Temp
;  call output_hex
;  movlw '\r'
;  call writechar
;  movlw '\n'
;  call writechar
;  movf Temp, w

; This is a kind of state machine.. wait for a number between 1 to 4
; (which server to move) after that wait for a value between a to z
; for the servo value

; Check to see if this is a parameter or servo_number

  movwf Temp
  movf next_servo, w
  xorlw 255 
  btfsc STATUS, Z 
  goto is_servo_number 

  ; got new value - change value on a servo
  movlw Servo_1
  addwf next_servo, w
  movwf FSR
  movf Temp, w
  movwf INDF

  movlw 255
  movwf next_servo

  movlw '<'
  call writechar
  movlw '\r'
  call writechar
  movlw '\n'
  call writechar

  goto MainLoop

is_servo_number:
  movf Temp, w
  ; sublw '1'
  addlw 207 
  movwf next_servo

  ; if next_servo > 4 this is out of range
  movf next_servo, w
  sublw 4
  btfss STATUS, C
  movlw '!'
  call writechar
  goto MainLoop

  movf next_servo, w
  ; put back in??
  ; call output_hex

; debug code
  movlw '?'
  call writechar
  movf Temp, w
  call writechar
  movlw '\r'
  call writechar
  movlw '\n'
  call writechar

  movlw 255
  movwf next_servo
  goto MainLoop

end




